<?php
require_once ASG_APPLICATION_PATH . '/services/AbstractService.php';

class Asg_Service_Affiliate extends Asg_Service_AbstractService
{
    const PLUGIN_NAMESPACE = 'Affiliate_Plugin';
    const HELPER_NAMESPACE = 'Affiliate_Helper';

    protected $_plugins = array();
    protected $_options = array();

    static public function isSingleton()
    {
        return TRUE;
    }

    public function generate()
    {
        error_reporting(E_ERROR|E_PARSE|E_CORE_ERROR|E_COMPILE_ERROR|E_USER_ERROR|E_RECOVERABLE_ERROR);
        ini_set('error_log', Asg_WpPlugin::getPluginPath() . '/error.txt');
        ini_set('log_errors', TRUE);
        require_once dirname(__FILE__) . '/Affiliate/ErrorHandler.php';
        Asg_Service_Affiliate_ErrorHandler::enable(E_ERROR|E_WARNING);

        $lockFile = fopen(__FILE__ . '.lock', 'r');
        if ($lockFile) {
            if (!flock($lockFile, LOCK_EX|LOCK_NB)) {
                exit('Another instance in running');
            }
        }

        set_time_limit(3600);
        ini_set('memory_limit', '512M');

        try {
            $this->feedback(NULL, 'progress');
            $this->ping();

            $publisher = $this->getPublisher();

            $maxItems = $this->getOption('max_items');
            $publishedCount = 0;
            $items = array();

            // Get item list
            $this->feedback('Reading product list... ');

            switch ($this->getOption('keywords_type')) {
                case 'asin':
                    // Generate items array from keywords list
                    foreach ((array) $this->getOption('search_keywords') as $key=>$keyword) {
                        if ($published = $publisher->isPublished($keyword)) {
                            $publishedCount++;
                        } else {
                            $items[$keyword] = array();
                        }
                    }

                    $maxItems = NULL;
                    $total = count($items);

                    break;

                case 'keyword':  // keyword is default type
                default:
                    $adapter = $this->getAdapter();

                    try {
                        $items = $adapter->getItems(round($maxItems
                            * (1 + $this->getOption('max_items_add_ratio'))));
                    } catch (Asg_Service_Affiliate_Exception $e) {
                        $this->feedback('Failed: ' . $e->getMessage(), 'error');
                        return;
                    }

                    $publishedCount = $adapter->getPublishedCount();

                    // Release adapter
                    $adapter = NULL;
                    $this->unloadPlugin('adapter');

                    $total = min(count($items), $maxItems);

                    break;
            }

            $count = count($items);

            if (0 == $count) {
                if ($publishedCount > 0) {
                    $this->feedback("Skipped {$publishedCount} product(s) already published.\n");
                    $this->feedback("No new products found for the search criteria. You already have posts for all products found.", 'error');
                } else {
                    $this->feedback("No products found for the search criteria.", 'error');
                }
                return;
            } else {
                $this->feedback("fetched {$count} product(s).\n");
                if ($publishedCount > 0) {
                    $this->feedback("Skipped {$publishedCount} product(s) already published.\n");
                }
            }

            $scrapers = $this->getPlugins('scraper');

            $processed = 0;
            $index = 0;

            global $wpdb;

            foreach ($items as $itemId=>$item) {
                $this->ping();
                $item['item_id'] = $itemId;

                $progress = ($total < $this->_maxItems)
                    ? $index / $total
                    : $processed / $total;

                $index++;

                $this->feedback("Generating post for product: {$this->getItemLabel($item)}... ", NULL, $progress);

                try {
                    foreach ($scrapers as $scraper) {
                        $scraper->scrape($item)
                            ->scrapeReviews($item);
                        $this->feedback();
                    }

                    if (!isset($item['text'])) {
                        $item['text'] = '';
                    }

                    // Clean null elements
                    foreach ($item as $key=>$value) {
                        if (is_null($value)) {
                            unset($item[$key]);
                        }
                    }

                    $publisher->publish($item);
                    $this->feedback();
                } catch (Asg_Service_Affiliate_Exception $e) {
                    $this->feedback('failed: ' . $e->getMessage() . "\n");
                    if ($e instanceof Asg_Service_Affiliate_FatalException) {
                        throw $e;
                    }
                    continue;
                }

                $this->feedback("ok\n");

                $processed++;
                if (!is_null($maxItems) && $processed >= $maxItems) {
                    break;
                }
            }
            $this->feedback("\n");

            $msg = $processed
                ? "{$processed} posts added successfully."
                : 'Unfortunately, found no product applicable to add a post.';
            $this->feedback("Finished. {$msg}", 'success', 1.0);
        } catch (Asg_Service_Affiliate_FatalException $e) {
            $this->feedback("\n\n{$e->getMessage()}\n\n", 'error');
            $this->feedback("\n\n");
        } catch (Asg_Service_Affiliate_CriticalException $e) {
            $this->feedback("\n\nCritical error: {$e->getMessage()}\n\n");
            $this->feedback("Debug trace: \n{$e->getTraceAsString()}\n\n");
            $this->feedback("\nCritical error occured. Please contact the support.", 'error');
            @error_log($e, E_ERROR);
        } catch (Exception $e) {
            $this->feedback("\n\nUnexpected error: {$e}\n\n");
            $this->feedback("Unexpected error occured. Please contact the support.", 'error');
            @error_log($e, E_ERROR);
        }

        Asg_Service_Affiliate_ErrorHandler::disable();

        fclose($lockFile);
    }

    public function refresh($publicationId, $itemId)
    {
        $item = array(
            'item_id' => $itemId,
        );

        try {
            foreach ($this->getScrapers() as $scraper) {
                $scraper->refresh($item);
            }
        } catch (Asg_Service_Affiliate_Exception $e) {
            $this->getPublisher()->refreshFailed($publicationId, $item);
            throw $e;
        }

        $this->getPublisher()->refresh($publicationId, $item);

        unset($item['item_id']);

        return $item;
    }

    public function feedback($message='', $status=NULL, $progress=NULL)
    {
        $feedback = $this->getFeedback();
        $command = $feedback->exchange($message, $status, $progress);

        if ('terminate' == $command) {
            $feedback->terminate();
            exit;
        }
    }

    /**
    * Workaround for rapidly disconnecting DB servers
    */
    public function ping()
    {
        @mysql_ping();
    }

    //
    // Helper functions
    //

    public function getItemLabel(array $item)
    {
        return (isset($item['title']) && '' != $item['title'])
            ? "{$item['title']} ({$item['item_id']})"
            : $item['item_id'];
    }

    /**
    * Throws exception
    *
    * @param string $message Error message
    * @param string $type Error type: fatal, critical or empty string
    */
    public function throwError($message, $type='')
    {
        $exceptionName = ucfirst($type) . 'Exception';
        require_once ASG_APPLICATION_PATH . "/services/Affiliate/{$exceptionName}.php";

        $exceptionClass = 'Asg_Service_Affiliate_' . $exceptionName;
        throw new $exceptionClass($message);
    }

    //
    // Getters & setters
    //

    public function getPlugin($type, $id='default')
    {
        if (!isset($this->_plugins[$type][$id])) {
            $config = Asg_Application::getConfig()->affiliate->plugin;
            if (!isset($this->_plugins[$type])) {
                $this->_plugins[$type] = array();
            }
            $class = $config->$type->$id;
            if (!$class) {
                throw new Exception("No config record for {$type}.{$id} plug-in");
            }

            return $this->_plugins[$type][$id] =
                Asg_Application::getService(self::PLUGIN_NAMESPACE . '_'
                    . ucfirst($type) . '_' . $class);
        }

        return $this->_plugins[$type][$id];
    }

    public function getPlugins($type)
    {
        $config = Asg_Application::getConfig()->affiliate->plugin->$type;

        $result = array();
        foreach ($config as $id=>$class) {
            $result[$id] = $this->getPlugin($type, $id);
        }

        return $result;
    }

    public function unloadPlugin($type, $id='default')
    {
        unset ($this->_plugins[$type][$id]);
    }

    public function unloadPlugins($type)
    {
        unset ($this->_plugins[$type]);
    }

    public function getAdapter()
    {
        return $this->getPlugin('adapter');
    }

    public function getScraper($id = 'main')
    {
        return $this->getPlugin('scraper', $id);
    }

     public function getScrapers()
    {
        return $this->getPlugins('scraper');
    }

    public function getPublisher()
    {
        return $this->getPlugin('publisher');
    }

    public function getFeedback()
    {
        return $this->getPlugin('feedback');
    }

    public function getHelper($name)
    {
        return Asg_Application::getService(self::HELPER_NAMESPACE . '_'
            . ucfirst($name));
    }

    public function getOption($key)
    {
        if (array_key_exists($key, $this->_options)) {
            return $this->_options[$key];
        }

        $result = Asg_Application::getConfig()->affiliate->options->$key;
        if (is_null($result)) {
            $this->throwError("Required option '{$key}' not found");
        }

        if ($result instanceof AsgZend_Config) {
            $result = $result->toArray();
        }

        return $result;
    }

    public function hasOption($key)
    {
        return isset($this->_options[$key]);
    }

    public function setOptions(array $options)
    {
        $this->_options = $options;
        return $this;
    }

    public function setOption($key, $value)
    {
        $this->_options[$key] = $value;
        return $this;
    }
}