<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Helper.php';

class Asg_Service_Affiliate_Helper_AmazonApi
    extends Asg_Service_Affiliate_Helper
{
    const THROTTLE_DELAY_SECONDS = 2;
    const THROTTLE_RETRY_COUNT = 5;

    protected $_market;
    protected $_config;

    /**
    * @var AsgZend_Http_Client
    */
    protected $_http;

    public function init()
    {
        parent::init();

        $this->_market = $this->getOption('amazon_market');
        // Create HTTP client
        $this->_http = $this->getHelper('Http')->getClient();
    }

    /**
     * Prepare and send signed AWS request
     * @param string $operation Operation name
     * @param array $params for AWS request
     * @returns SimpleXml object AWS response
     */
    public function request($operation, array $params)
    {
        // Main parametrs
        $method = 'GET';
        $host = $this->getMarketConfig()->apiHost;
        $uri = '/onca/xml';

        $iRetry = 0;
        while(true) {
            // additional parameters
            $params['Service'] = 'AWSECommerceService';
            $params['AWSAccessKeyId'] = $this->getOption('amazon_key');
            $params['AssociateTag'] = $this->hasOption('affiliate_id')
                ? $this->getOption('affiliate_id')
                : '-';
            $params['Timestamp'] = gmdate('Y-m-d\TH:i:s\Z', time() + 24*3600); //workaround for servers with late clock
            $params['Version'] = '2011-08-01';
            $params['Operation'] = $operation;

            // prepare & sign request
            ksort($params);
            $canonicalizedQuery = array();
            foreach ($params as $param=>$value) {
                $param = str_replace('%7E', '~', rawurlencode($param));
                $value = str_replace('%7E', '~', rawurlencode($value));
                $canonicalizedQuery[] = $param . '=' . $value;
            }
            $canonicalizedQuery = implode('&', $canonicalizedQuery);
            $stringToSign = $method . "\n" . $host . "\n" . $uri . "\n" . $canonicalizedQuery;
            $signature = base64_encode(hash_hmac('sha256', $stringToSign, $this->getOption('amazon_secret_key'), TRUE));
            $signature = str_replace('%7E', '~', rawurlencode($signature));

            $scheme = $this->getOption('product_advertising_api_secure')? 'https' : 'http';
            $url = "{$scheme}://{$host}{$uri}?{$canonicalizedQuery}&Signature={$signature}";

            // do request
            try {
                $response = $this->_http->setUri($url)->request($method);
            } catch (AsgZend_Http_Client_Exception $e) {
                $this->throwError("Cannot connect to Amazon API ({$e->getMessage()})", 'fatal');
            } catch (Exception $e) {
                $this->throwError("Unexpected error while requesting Amazon API ({$e->getMessage()})", 'fatal');
            }
            if ($response->isError()) {
                if (403 == $response->getStatus()) {
                    $this->throwError('Invalid Amazon credentials', 'fatal');
                }

                $xml = @simplexml_load_string($response->getBody());

                if (isset($xml->Error->Code)
                    && 'RequestThrottled' == $xml->Error->Code
                    && $iRetry < self::THROTTLE_RETRY_COUNT
                ) {
                    $iRetry++;
                    sleep(self::THROTTLE_DELAY_SECONDS);
                    continue;
                }

                if (isset($xml->Error->Message)) {
                    $this->throwError("Amazon API returned error: {$xml->Error->Message}", 'fatal');
                }

                $this->throwError("Amazon API request  failed due to HTTP error #{$response->getStatus()} ({$response->getMessage()})", 'fatal');
            }

            break;
        }

        $response = @simplexml_load_string($response->getBody());
        if (FALSE === $response) {
            $this->throwError('Error parsing API response', 'fatal');
        }

        return $response;
    }

    //
    // Getters & setters
    //

    public function getMarket()
    {
        return $this->_market;
    }

    public function getMarketConfig()
    {
        if (!$this->_config) {
            require_once ASG_LIBRARY_PATH . '/AsgZend/Config/Ini.php';
            $this->_config = new AsgZend_Config_Ini(ASG_APPLICATION_PATH . '/configs/amazon.ini', $this->_market);
        }

        return $this->_config;
    }
}