<?php
require_once ASG_APPLICATION_PATH . '/services/Affiliate/Plugin/Adapter.php';

class Asg_Service_Affiliate_Plugin_Adapter_Amazon
    extends Asg_Service_Affiliate_Plugin_Adapter
{
    /**
    * @var Asg_Service_Affiliate_Helper_AmazonApi
    */
    protected $_apiHelper;

    public function init()
    {
        parent::init();

        $this->_apiHelper = $this->getHelper('AmazonApi');
    }

    public function getPriceRanges()
    {
        return $this->_apiHelper->getMarketConfig()->priceRange->toArray();
    }

    public function getCategories($parent = NULL)
    {
        $cacheKey = "amazon.categories.{$this->getOption('amazon_market')}."
            . (is_null($parent)? '_root' : $parent);
        /**
        * @var Asg_Model_Cache
        */
        $cache = Asg_Application::getModel('Cache');
        $result = $cache->getValue($cacheKey);
        if (!is_null($result)) {
            return $result;
        }

        if (is_null($parent)) {
            // Return root categories
            $result = $this->_apiHelper->getMarketConfig()->rootCategories
                ->toArray();
        } else {
            $result = $this->_getSubcategories($parent);
        }

        natcasesort($result);

        $cache->setValue($cacheKey, $result)
            ->save();

        return $result;
    }

    /**
    * Get array of subcategories with & replace 'and'
    * @param int $parent
    * @return array: key - nodeId, value - name
    */
    protected function _getSubcategories($parent)
    {
        $result = array();
        $response = $this->_apiHelper->request('BrowseNodeLookup', array(
            'BrowseNodeId' => $parent,
        ));

        if (isset($response->BrowseNodes->BrowseNode->Children->BrowseNode)) {
            $nodes = $response->BrowseNodes->BrowseNode->Children->BrowseNode;

            // Try to find nodes with IsCategoryRoot
            $traverse = array();
            foreach ($nodes as $key=>$node) {
                if (isset($node->IsCategoryRoot) && $node->IsCategoryRoot) {
                    $traverse[$key] = $node;
                }
            }

            if (!$traverse) {
                foreach ($nodes as $key=>$node ) {
                    $result[(string)$node->BrowseNodeId] = (string)$node->Name;
                }
            } else {
                foreach ($traverse as $key=>$node) {
                    $response = $this->_apiHelper->request('BrowseNodeLookup', array(
                        'BrowseNodeId' => $node->BrowseNodeId)
                    );
                    if (isset($response->BrowseNodes->BrowseNode->Children->BrowseNode) ) {
                        foreach ($response->BrowseNodes->BrowseNode->Children->BrowseNode as $subKey=>$subNode ) {
                            $result[(string)$subNode->BrowseNodeId] = (string)$subNode->Name;
                        }
                    }
                }
            }
        }

        return $result;
    }

    /**
     * Get items for array keywords
     */
    public function getItems($count, $preview=FALSE)
    {
        // Set keywords
        $searchKeywords = $this->getOption('search_keywords');
        if ($searchKeywords) {
            foreach ($searchKeywords as $key=>$keyword) {
                $keywords[$key] = '"' . $keyword . '"';
            }
            $params  = array(
                'Keywords' => implode(' ', $keywords),
            );
        }

        // Set category to search
        $category = $this->getOption('search_category');
        if ($category) {
            $params['BrowseNode'] = $category;
        }

        //Set price filter
        $useAltSearchIndex = false;
        if ($this->hasOption('price_min')) {
            $params['MinimumPrice'] = $this->getOption('price_min');
            $useAltSearchIndex = TRUE;
        }
        if ($this->hasOption('price_max')) {
            $params['MaximumPrice'] = $this->getOption('price_max');
            $useAltSearchIndex = TRUE;
        }

        // Set search index
        $rootCategory = $this->getOption('root_category');
        if (!$rootCategory) {
            $rootCategory = 'all';
        }
        $catCfg = $this->_apiHelper->getMarketConfig()->categoryConfig->$rootCategory;
        $params['SearchIndex'] = ($useAltSearchIndex && isset($catCfg->altSearchIndex))
            ? $catCfg->altSearchIndex
            : $catCfg->searchIndex;

        $result = array();

        $this->_getItems($count, $preview, $params, $result);

        if (!$result && !$this->_publishedCount
            && count($searchKeywords) > 1
        ) {
            // Iterate through keywords
            foreach ($searchKeywords as $keyword) {
                $params['Keywords'] = $keyword;
                $this->_getItems($count, $preview, $params, $result);
                if (($preview ? count($result) - $this->_publishedCount : count($result)) >= $count) {
                    break;
                }
            }
        }

        return $result;
    }

    /**
     * Get items for simple keyword
     */
    protected function _getItems($count, $preview, $params, &$result)
    {
        // Get publisher to check exisiting items:
        $srv = $this->getMainService();
        $publisher = $srv->getPublisher();
        // Start page loop
        $currentPage = 0;
        while (($preview ? count($result) - $this->_publishedCount : count($result)) < $count) {
            $currentPage++;

            $params['ItemPage'] = $currentPage;

            try {
                $response = $this->_apiHelper->request('ItemSearch', $params);
            } catch (Asg_Service_Affiliate_Exception $e) {
                // Make it fatal error
                $this->throwError($e->getMessage());
            }

            $srv->ping();

            if (!isset($response->Items->Item)
                || !count($response->Items->Item)
            ) {
                if (1 == $currentPage
                    && isset($response->Items->Request->Errors->Error->Code)
                    && 'AWS.ECommerceService.NoExactMatches' != $response->Items->Request->Errors->Error->Code
                ) {
                    $msg = isset($response->Items->Request->Errors->Error->Message)
                        ? (string) $response->Items->Request->Errors->Error->Message
                        : (string) $response->Items->Request->Errors->Error->Code;
                    $this->throwError('Amazon API returned error: ' . $msg, 'fatal');
                }
                break;
            }

            foreach ($response->Items->Item as $itemData) {
                $id = (string) $itemData->ASIN;

                // Filter out existing items
                if ($published = $publisher->isPublished($id)) {
                    $this->_publishedCount++;
                    if (!$preview) {
                        continue;
                    }
                }

                $result[$id] = array(
                    'title' => (string) $itemData->ItemAttributes->Title,
                    'amazon_market' => $this->getOption('amazon_market'),
                );
                if ($preview) {
                    $result[$id]['published'] = $published;
                    if (count($result) - $this->_publishedCount >= $count) {
                        break;
                    }
                }
            }

            if ($currentPage >= $response->Items->TotalPages) {
                break;
            }
        }

        return $result;
    }
}